import axios from 'axios'
import { Message } from 'element-ui'
import Vue from 'vue'

// 启动开发服务时，webpack 会设置 process.env.NODE_ENV 为 development
const urlPrefix = {
  development: 'http://localhost:3000',
  production: '/'
}

const baseURL = urlPrefix[process.env.NODE_ENV]

const instance = axios.create({
  baseURL,
  timeout: 10000,
  responseType: 'json',
  withCredentials: true,
  headers: { // 通用请求头
    'Content-Type': 'application/json;charset=utf-8' // get 请求不生效，post 请求才生效
  }
})

// instance.defaults.headers.common['xxx-common'] = 'aaa' // 所有请求类型均包含的请求头
// instance.defaults.headers.post.abc = 'aabbcc' // post 请求包含的请求头
// instance.defaults.headers.get.efg = 'eeffgg' // get 请求包含的请求头

instance.interceptors.request.use((config) => {
  config.headers.authorization = localStorage.getItem('token')
  return config
}, (err) => {
  // 【存疑】何时调用该回调？
  Promise.reject(err)
})

// 不管成功还是失败的回调，返回值都是 promise.resolve 出来，除非返回 Promise.reject
instance.interceptors.response.use(({ data }) => {
  if (data.code === 0) {
    return data
  }

  Message.error(data.data)
  return Promise.reject(data)
}, err => Promise.reject(err))

class AjaxRequest {
  constructor() {
    this.xhrQueue = {}
  }

  queueCleaner(key) {
    if (!Reflect.has(this.xhrQueue, key)) return
    return Reflect.deleteProperty(this.xhrQueue, key)
  }

  clearQueue() {
    Object.entries(this.xhrQueue).forEach(([key, item]) => {
      item.source.cancel()
      Reflect.deleteProperty(this.xhrQueue, key)
    })
  }

  request(method, url, params = null, options = {}) {
    // Vue.prototype.$store.commit('setLoading', true)
    const xhrKey = JSON.stringify({ url, params, method })
    if (Reflect.has(this.xhrQueue, xhrKey)) {
      return this.xhrQueue[xhrKey].xhr
    }

    const { CancelToken } = axios
    const source = CancelToken.source()

    const config = {
      method,
      url,
      [method === 'get' ? 'params' : 'data']: params,
      cancelToken: source.token,
      ...options
    }

    const xhr = instance(config).finally(() => {
      // Vue.prototype.$store.commit('setLoading', false)
      this.queueCleaner(xhrKey)
    })
    this.xhrQueue[xhrKey] = { source, xhr }
    return xhr
  }

  get(url, params, options) {
    return this.request('get', url, params, options)
  }

  post(url, params, options) {
    return this.request('post', url, params, options)
  }
}

const AjaxService = new AjaxRequest()
Vue.prototype.AjaxService = AjaxService
export default AjaxService
